@@ -1,19 +1,37 @@
 """Catch moment when tables are in sync on master and slave.
 """
 
-from __future__ import division, absolute_import, print_function
-
-import sys
-import time
+import logging
 import os
 import subprocess
+import sys
+import time
+from typing import IO, List, Optional, Sequence, Tuple, Dict, cast, Mapping, Any
 
 import skytools
 
-class TableRepair(object):
+from .basetypes import Connection, Cursor
+
+DictRow = Dict[str, str]
+
+
+class TableRepair:
     """Checks that tables in two databases are in sync."""
 
-    def __init__(self, table_name, log):
+    table_name: str
+    fq_table_name: str
+    log: logging.Logger
+    pkey_list: List[str]
+    common_fields: List[str]
+    apply_fixes: bool
+    apply_cursor: Optional[Cursor]
+    cnt_insert: int
+    cnt_update: int
+    cnt_delete: int
+    total_src: int
+    total_dst: int
+
+    def __init__(self, table_name: str, log: logging.Logger) -> None:
         self.table_name = table_name
         self.fq_table_name = skytools.quote_fqident(table_name)
         self.log = log
@@ -25,7 +43,7 @@ class TableRepair(object):
 
         self.reset()
 
-    def reset(self):
+    def reset(self) -> None:
         self.cnt_insert = 0
         self.cnt_update = 0
         self.cnt_delete = 0
@@ -36,7 +54,8 @@ class TableRepair(object):
         self.apply_fixes = False
         self.apply_cursor = None
 
-    def do_repair(self, src_db, dst_db, where, pfx='repair', apply_fixes=False):
+    def do_repair(self, src_db: Connection, dst_db: Connection, where: str,
+                  pfx: str = 'repair', apply_fixes: bool = False) -> None:
         """Actual comparison."""
 
         self.reset()
@@ -79,27 +98,26 @@ class TableRepair(object):
         if apply_fixes:
             dst_db.commit()
 
-    def do_sort(self, src, dst):
-        p = subprocess.Popen(["sort", "--version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        s_ver = p.communicate()[0]
-        del p
+    def do_sort(self, src: str, dst: str) -> None:
+        with subprocess.Popen(["sort", "--version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE) as p:
+            s_ver = p.communicate()[0]
 
         xenv = os.environ.copy()
         xenv['LANG'] = 'C'
         xenv['LC_ALL'] = 'C'
 
         cmdline = ['sort', '-T', '.']
-        if s_ver.find("coreutils") > 0:
+        if s_ver.find(b"coreutils") > 0:
             cmdline.append('-S')
             cmdline.append('30%')
         cmdline.append('-o')
         cmdline.append(dst)
         cmdline.append(src)
-        p = subprocess.Popen(cmdline, env=xenv)
-        if p.wait() != 0:
-            raise Exception('sort failed')
+        with subprocess.Popen(cmdline, env=xenv) as p:
+            if p.wait() != 0:
+                raise Exception('sort failed')
 
-    def gen_copy_tbl(self, src_curs, dst_curs, where):
+    def gen_copy_tbl(self, src_curs: Cursor, dst_curs: Cursor, where: str) -> str:
         """Create COPY expession from common fields."""
         self.pkey_list = skytools.get_table_pkeys(src_curs, self.table_name)
         dst_pkey = skytools.get_table_pkeys(dst_curs, self.table_name)
@@ -131,32 +149,36 @@ class TableRepair(object):
 
         return tbl_expr
 
-    def dump_table(self, copy_cmd, curs, fn):
+    def dump_table(self, copy_cmd: str, curs: Cursor, fn: str) -> None:
         """Dump table to disk."""
-        f = open(fn, "w", 64*1024)
-        curs.copy_expert(copy_cmd, f)
-        self.log.info('%s: Got %d bytes', self.table_name, f.tell())
-        f.close()
+        with open(fn, "w", 64 * 1024, encoding="utf8") as f:
+            curs.copy_expert(copy_cmd, f)
+            self.log.info('%s: Got %d bytes', self.table_name, f.tell())
 
-    def get_row(self, ln):
+    def get_row(self, ln: str) -> Optional[DictRow]:
         """Parse a row into dict."""
         if not ln:
             return None
         t = ln[:-1].split('\t')
-        row = {}
-        for i in range(len(self.common_fields)):
-            row[self.common_fields[i]] = t[i]
+        row: DictRow = {}
+        for i, n in enumerate(self.common_fields):
+            row[n] = t[i]
         return row
 
-    def dump_compare(self, src_fn, dst_fn, fix):
+    def dump_compare(self, src_fn: str, dst_fn: str, fix: str) -> None:
         """Dump + compare single table."""
         self.log.info("Comparing dumps: %s", self.table_name)
-        f1 = open(src_fn, "r", 64*1024)
-        f2 = open(dst_fn, "r", 64*1024)
+        with open(src_fn, "r", 64 * 1024, encoding="utf8") as f1:
+            with open(dst_fn, "r", 64 * 1024, encoding="utf8") as f2:
+                self.dump_compare_streams(f1, f2, fix)
+
+    def dump_compare_streams(self, f1: IO[str], f2: IO[str], fix: str) -> None:
         src_ln = f1.readline()
         dst_ln = f2.readline()
-        if src_ln: self.total_src += 1
-        if dst_ln: self.total_dst += 1
+        if src_ln:
+            self.total_src += 1
+        if dst_ln:
+            self.total_dst += 1
 
         if os.path.isfile(fix):
             os.unlink(fix)
@@ -168,31 +190,35 @@ class TableRepair(object):
                 dst_row = self.get_row(dst_ln)
 
                 diff = self.cmp_keys(src_row, dst_row)
-                if diff > 0:
+                if diff > 0 and dst_row:
                     # src > dst
                     self.got_missed_delete(dst_row, fix)
                     keep_src = 1
-                elif diff < 0:
+                elif diff < 0 and src_row:
                     # src < dst
                     self.got_missed_insert(src_row, fix)
                     keep_dst = 1
-                else:
+                elif src_row and dst_row:
                     if self.cmp_data(src_row, dst_row) != 0:
                         self.got_missed_update(src_row, dst_row, fix)
 
             if not keep_src:
                 src_ln = f1.readline()
-                if src_ln: self.total_src += 1
+                if src_ln:
+                    self.total_src += 1
             if not keep_dst:
                 dst_ln = f2.readline()
-                if dst_ln: self.total_dst += 1
+                if dst_ln:
+                    self.total_dst += 1
 
-        self.log.info("finished %s: src: %d rows, dst: %d rows,"\
-                    " missed: %d inserts, %d updates, %d deletes",
-                self.table_name, self.total_src, self.total_dst,
-                self.cnt_insert, self.cnt_update, self.cnt_delete)
+        self.log.info("finished %s: src: %d rows, dst: %d rows,"
+                      " missed: %d inserts, %d updates, %d deletes",
+                      self.table_name, self.total_src, self.total_dst,
+                      self.cnt_insert, self.cnt_update, self.cnt_delete)
+        f1.close()
+        f2.close()
 
-    def got_missed_insert(self, src_row, fn):
+    def got_missed_insert(self, src_row: DictRow, fn: str) -> None:
         """Create sql for missed insert."""
         self.cnt_insert += 1
         fld_list = self.common_fields
@@ -203,15 +229,15 @@ class TableRepair(object):
             v = skytools.unescape_copy(src_row[f])
             val_list.append(skytools.quote_literal(v))
         q = "insert into %s (%s) values (%s);" % (
-                self.fq_table_name, ", ".join(fq_list), ", ".join(val_list))
+            self.fq_table_name, ", ".join(fq_list), ", ".join(val_list))
         self.show_fix(q, 'insert', fn)
 
-    def got_missed_update(self, src_row, dst_row, fn):
+    def got_missed_update(self, src_row: DictRow, dst_row: DictRow, fn: str) -> None:
         """Create sql for missed update."""
         self.cnt_update += 1
         fld_list = self.common_fields
-        set_list = []
-        whe_list = []
+        set_list: List[str] = []
+        whe_list: List[str] = []
         for f in self.pkey_list:
             self.addcmp(whe_list, skytools.quote_ident(f), skytools.unescape_copy(src_row[f]))
         for f in fld_list:
@@ -224,33 +250,34 @@ class TableRepair(object):
             self.addcmp(whe_list, skytools.quote_ident(f), skytools.unescape_copy(v2))
 
         q = "update only %s set %s where %s;" % (
-                self.fq_table_name, ", ".join(set_list), " and ".join(whe_list))
+            self.fq_table_name, ", ".join(set_list), " and ".join(whe_list))
         self.show_fix(q, 'update', fn)
 
-    def got_missed_delete(self, dst_row, fn):
+    def got_missed_delete(self, dst_row: DictRow, fn: str) -> None:
         """Create sql for missed delete."""
         self.cnt_delete += 1
-        whe_list = []
+        whe_list: List[str] = []
         for f in self.pkey_list:
             self.addcmp(whe_list, skytools.quote_ident(f), skytools.unescape_copy(dst_row[f]))
         q = "delete from only %s where %s;" % (self.fq_table_name, " and ".join(whe_list))
         self.show_fix(q, 'delete', fn)
 
-    def show_fix(self, q, desc, fn):
+    def show_fix(self, q: str, desc: str, fn: str) -> None:
         """Print/write/apply repair sql."""
         self.log.debug("missed %s: %s", desc, q)
-        open(fn, "a").write("%s\n" % q)
+        with open(fn, "a", encoding="utf8") as f:
+            f.write("%s\n" % q)
 
-        if self.apply_fixes:
+        if self.apply_fixes and self.apply_cursor:
             self.apply_cursor.execute(q)
 
-    def addeq(self, dst_list, f, v):
+    def addeq(self, dst_list: List[str], f: str, v: Optional[str]) -> None:
         """Add quoted SET."""
         vq = skytools.quote_literal(v)
         s = "%s = %s" % (f, vq)
         dst_list.append(s)
 
-    def addcmp(self, dst_list, f, v):
+    def addcmp(self, dst_list: List[str], f: str, v: Optional[str]) -> None:
         """Add quoted comparison."""
         if v is None:
             s = "%s is null" % f
@@ -259,7 +286,7 @@ class TableRepair(object):
             s = "%s = %s" % (f, vq)
         dst_list.append(s)
 
-    def cmp_data(self, src_row, dst_row):
+    def cmp_data(self, src_row: DictRow, dst_row: DictRow) -> int:
         """Compare data field-by-field."""
         for k in self.common_fields:
             v1 = src_row[k]
@@ -268,7 +295,7 @@ class TableRepair(object):
                 return -1
         return 0
 
-    def cmp_value(self, v1, v2):
+    def cmp_value(self, v1: str, v2: str) -> int:
         """Compare single field, tolerates tz vs notz dates."""
         if v1 == v2:
             return 0
@@ -287,7 +314,7 @@ class TableRepair(object):
 
         return -1
 
-    def cmp_keys(self, src_row, dst_row):
+    def cmp_keys(self, src_row: Optional[DictRow], dst_row: Optional[DictRow]) -> int:
         """Compare primary keys of the rows.
 
         Returns 1 if src > dst, -1 if src < dst and 0 if src == dst"""
@@ -312,11 +339,12 @@ class TableRepair(object):
 
 class Syncer(skytools.DBScript):
     """Checks that tables in two databases are in sync."""
-    lock_timeout = 10
-    ticker_lag_limit = 20
-    consumer_lag_limit = 20
+    lock_timeout: float = 10
+    ticker_lag_limit: int = 20
+    consumer_lag_limit: int = 20
 
-    def sync_table(self, cstr1, cstr2, queue_name, consumer_name, table_name):
+    def sync_table(self, cstr1: str, cstr2: str, queue_name: str,
+                   consumer_name: str, table_name: str) -> Tuple[Connection, Connection]:
         """Syncer main function.
 
         Returns (src_db, dst_db) that are in transaction
@@ -355,7 +383,7 @@ class Syncer(skytools.DBScript):
         setup_curs.execute("select to_char(now(), 'YYYY-MM-DD HH24:MI:SS.MS')")
         tpos = setup_curs.fetchone()[0]
         # now wait
-        while 1:
+        while True:
             time.sleep(0.5)
 
             q = "select now() - lag > timestamp %s, now(), lag from pgq.get_consumer_info(%s, %s)"
@@ -390,14 +418,14 @@ class Syncer(skytools.DBScript):
 
         return (src_db, dst_db)
 
-    def set_lock_timeout(self, curs):
+    def set_lock_timeout(self, curs: Cursor) -> None:
         ms = int(1000 * self.lock_timeout)
         if ms > 0:
             q = "SET LOCAL statement_timeout = %d" % ms
             self.log.debug(q)
             curs.execute(q)
 
-    def check_consumer(self, curs, queue_name, consumer_name):
+    def check_consumer(self, curs: Cursor, queue_name: str, consumer_name: str) -> None:
         """ Before locking anything check if consumer is working ok.
         """
         self.log.info("Queue: %s Consumer: %s", queue_name, consumer_name)
@@ -425,7 +453,7 @@ class Syncer(skytools.DBScript):
             self.log.error('Consumer lagging too much, cannot proceed')
             sys.exit(1)
 
-    def force_tick(self, curs, queue_name):
+    def force_tick(self, curs: Cursor, queue_name: str) -> None:
         """ Force tick into source queue so that consumer can move on faster
         """
         q = "select pgq.force_tick(%s)"
@@ -434,7 +462,7 @@ class Syncer(skytools.DBScript):
         cur_pos = res[0]
 
         start = time.time()
-        while 1:
+        while True:
             time.sleep(0.5)
             curs.execute(q, [queue_name])
             res = curs.fetchone()
@@ -501,9 +529,9 @@ class Checker(Syncer):
                and q.queue_name like 'xm%%%%'
     """
 
-    def __init__(self, args):
+    def __init__(self, args: Sequence[str]) -> None:
         """Checker init."""
-        super(Checker, self).__init__('data_checker', args)
+        super().__init__('data_checker', args)
         self.set_single_loop(1)
         self.log.info('Checker starting %s', str(args))
 
@@ -511,7 +539,7 @@ class Checker(Syncer):
 
         self.table_list = self.cf.getlist('table_list')
 
-    def work(self):
+    def work(self) -> Optional[int]:
         """Syncer main function."""
 
         source_query = self.cf.get('source_query')
@@ -530,12 +558,13 @@ class Checker(Syncer):
             s_host = src_row['hostname']
             s_db = src_row['db_name']
 
-            curs.execute(consumer_query, src_row)
+            cast_row = cast(Mapping[str, Any], src_row)
+            curs.execute(consumer_query, cast_row)
             r = curs.fetchone()
             consumer_name = r['consumer_name']
             queue_name = r['queue_name']
 
-            curs.execute(target_query, src_row)
+            curs.execute(target_query, cast_row)
             for dst_row in curs.fetchall():
                 d_db = dst_row['db_name']
                 d_host = dst_row['hostname']
@@ -545,7 +574,7 @@ class Checker(Syncer):
                 where = where_expr % dst_row
 
                 self.log.info('Source: db=%s host=%s queue=%s consumer=%s',
-                                  s_db, s_host, queue_name, consumer_name)
+                              s_db, s_host, queue_name, consumer_name)
                 self.log.info('Target: db=%s host=%s where=%s', d_db, d_host, where)
 
                 for tbl in self.table_list:
@@ -553,21 +582,22 @@ class Checker(Syncer):
                     if check == 'compare':
                         self.do_compare(tbl, src_db, dst_db, where)
                     elif check == 'repair':
-                        r = TableRepair(tbl, self.log)
-                        r.do_repair(src_db, dst_db, where, 'fix.' + tbl, False)
+                        tr = TableRepair(tbl, self.log)
+                        tr.do_repair(src_db, dst_db, where, 'fix.' + tbl, False)
                     elif check == 'repair-apply':
-                        r = TableRepair(tbl, self.log)
-                        r.do_repair(src_db, dst_db, where, 'fix.' + tbl, True)
+                        tr = TableRepair(tbl, self.log)
+                        tr.do_repair(src_db, dst_db, where, 'fix.' + tbl, True)
                     elif check == 'compare-repair-apply':
                         ok = self.do_compare(tbl, src_db, dst_db, where)
                         if not ok:
-                            r = TableRepair(tbl, self.log)
-                            r.do_repair(src_db, dst_db, where, 'fix.' + tbl, True)
+                            tr = TableRepair(tbl, self.log)
+                            tr.do_repair(src_db, dst_db, where, 'fix.' + tbl, True)
                     else:
                         raise Exception('unknown check type')
                     self.reset()
+        return None
 
-    def do_compare(self, tbl, src_db, dst_db, where):
+    def do_compare(self, tbl: str, src_db: Connection, dst_db: Connection, where: str) -> bool:
         """Actual comparison."""
 
         src_curs = src_db.cursor()
@@ -575,7 +605,7 @@ class Checker(Syncer):
 
         self.log.info('Counting %s', tbl)
 
-        q = "select count(1) as cnt, sum(hashtext(t.*::text)) as chksum from only _TABLE_ t where %s;" %  where
+        q = "select count(1) as cnt, sum(hashtext(t.*::text)) as chksum from only _TABLE_ t where %s;" % where
         q = self.cf.get('compare_sql', q)
         q = q.replace('_TABLE_', skytools.quote_fqident(tbl))
 
