@@ -5,15 +5,15 @@ import skytools
 from skytools import skylog
 
 
-def test_trace_setup():
+def test_trace_setup() -> None:
     assert skylog.TRACE < logging.DEBUG
-    assert skylog.TRACE == logging.TRACE
+    assert skylog.TRACE == logging.TRACE    # type: ignore
     assert logging.getLevelName(skylog.TRACE) == "TRACE"
 
 
-def test_skylog():
+def test_skylog() -> None:
     log = skytools.getLogger("test.skylog")
     log.trace("tracemsg")
 
-    assert not log.isEnabledFor(logging.TRACE)
+    assert not log.isEnabledFor(logging.TRACE)  # type: ignore
 
